#!/usr/bin/perl

chdir "source";
@list = glob "*.occ";
foreach $file (@list)
  {
  # Redirect stdout and stderr to the null device.
  # This will prevent errors from the tests from messing up the screen.
  open(SAVEOUT,">&STDOUT");
  open(SAVEERR,">&STDERR");
  open(STDOUT,"/dev/null");
  open(STDERR,"/dev/null");

  $error="";
  $error="File 'occam/$file' not found." if (!-e "../occam/".$file);

  # Compile the source program, and compare it to the stored program.
  if ($error eq "")
    {
    $rt = system "occ2c -i -mw -posz -o temp1.occ $file";
    $error="Compilation of 'source/$file' to 'temp1.occ' failed." if ($rt!=0);
    }
  if ($error eq "")
    {
    $rt = system "diff -q temp1.occ ../occam/$file";
    $error="Difference detected between 'occam/$file' and 'temp1.occ'." if ($rt!=0);
    }

  # Compile temp1.occ from above, without optimisation.
  if ($error eq "")
    {
    $rt = system "occ2c -i -mw -pos -o temp2.occ temp1.occ";
    $error="Compilation of 'temp1.occ' to 'temp2.occ' failed." if ($rt!=0);
    }
  if ($error eq "")
    {
    $rt = system "diff -q temp1.occ temp2.occ";
    $error="Difference detected between 'temp1.occ' and 'temp2.occ'." if ($rt!=0);
    }

  # Check that the symbol tables for the source program, and the stored program
  # match each other.
  if ($error eq "")
    {
    $rt = system "occ2c -i -mw -pcsz -zs $file > temp1.sym";
    $error="Could not get symbol table for 'source/$file'." if ($rt!=0);
    }
  if ($error eq "")
    {
    $rt = system "occ2c -i -mw -pcs -zs ../occam/$file > temp2.sym";
    $error="Could not get symbol table for 'occam/$file'." if ($rt!=0);
    }
  if ($error eq "")
    {
    $rt = system "diff -q temp1.sym temp2.sym";
    $error="Symbol tables differ for 'source/$file' and 'occam/$file'." if ($rt!=0);
    }

  # Print a message, if an error has occured.
  if ($error ne "")
    {
    print SAVEOUT "Test failed ($file):\n";
    print SAVEOUT "\t$error\n";
    print SAVEOUT "\n";
    }

  unlink "temp1.occ";
  unlink "temp2.occ";
  unlink "temp1.sym";
  unlink "temp2.sym";

  # Restore stdout and stderr to their original states.
  close(STDOUT);
  close(STDERR);
  open(STDOUT,">&SAVEOUT");
  open(STDERR,">&SAVEERR");
  }
